@@ -42,6 +42,35 @@ class FunctionResolveTest extends TestCase
             );
     }
 
+    /** @test */
+    public function shouldResolveAThenable()
+    {
+        $thenable = new SimpleFulfilledTestThenable();
+
+        $mock = $this->createCallableMock();
+        $mock
+            ->expects($this->once())
+            ->method('__invoke')
+            ->with($this->identicalTo('foo'));
+
+        resolve($thenable)
+            ->then(
+                $mock,
+                $this->expectCallableNever()
+            );
+    }
+
+    /** @test */
+    public function shouldResolveACancellableThenable()
+    {
+        $thenable = new SimpleTestCancellableThenable();
+
+        $promise = resolve($thenable);
+        $promise->cancel();
+
+        $this->assertTrue($thenable->cancelCalled);
+    }
+
     /** @test */
     public function shouldRejectARejectedPromise()
     {
@@ -92,6 +121,44 @@ class FunctionResolveTest extends TestCase
         $result->then($mock);
     }
 
+    /** @test */
+    public function shouldSupportVeryDeepNestedPromises()
+    {
+        $deferreds = [];
+
+        // @TODO Increase count once global-queue is merged
+        for ($i = 0; $i < 10; $i++) {
+            $deferreds[] = $d = new Deferred();
+            $p = $d->promise();
+
+            $last = $p;
+            for ($j = 0; $j < 10; $j++) {
+                $last = $last->then(function($result) {
+                    return $result;
+                });
+            }
+        }
+
+        $p = null;
+        foreach ($deferreds as $d) {
+            if ($p) {
+                $d->resolve($p);
+            }
+
+            $p = $d->promise();
+        }
+
+        $deferreds[0]->resolve(true);
+
+        $mock = $this->createCallableMock();
+        $mock
+            ->expects($this->once())
+            ->method('__invoke')
+            ->with($this->identicalTo(true));
+
+        $deferreds[0]->promise()->then($mock);
+    }
+
     /** @test */
     public function returnsExtendePromiseForSimplePromise()
     {
